using System.Threading.Tasks;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using NSubstitute;
using NSubstitute.ExceptionExtensions;
using Robware.Api.Blog.Controllers;
using Robware.Blog;
using Robware.Data;
using Xunit;

namespace Robware.Api.Blog.Tests {
	public class BlogControllerTests {
		[Fact]
		public async Task Get_WithUrl_ReturnsBlogPost() {
			var logger = Substitute.For<ILogger<BlogController>>();
			var repo = Substitute.For<IBlogRepository>();

			repo.GetPostByUrlAsync("url").Returns(new BlogPost());

			var expectation = new BlogPost();

			var controller = new BlogController(logger, repo);
			(await controller.Get("url")).Value.Should().BeEquivalentTo(expectation);
		}

		[Fact]
		public async Task Get_WithId_ReturnsBlogPost() {
			var logger = Substitute.For<ILogger<BlogController>>();
			var repo = Substitute.For<IBlogRepository>();

			repo.GetPostByIdAsync(1).Returns(new BlogPost());

			var expectation = new BlogPost();

			var controller = new BlogController(logger, repo);
			(await controller.Get("1")).Value.Should().BeEquivalentTo(expectation);
		}

		[Fact]
		public async Task Get_WithInvalidUrl_Returns404() {
			var logger = Substitute.For<ILogger<BlogController>>();
			var repo = Substitute.For<IBlogRepository>();

			repo.GetPostByUrlAsync("url").Throws(new ItemNotFoundException("", null));

			var controller = new BlogController(logger, repo);
			(await controller.Get("url")).Result.Should().BeOfType<NotFoundObjectResult>();
		}

		[Fact]
		public async Task Get_WithInvalidId_Returns404() {
			var logger = Substitute.For<ILogger<BlogController>>();
			var repo = Substitute.For<IBlogRepository>();

			repo.GetPostByIdAsync(1).Throws(new ItemNotFoundException("", null));

			var controller = new BlogController(logger, repo);
			(await controller.Get("1")).Result.Should().BeOfType<NotFoundObjectResult>();
		}

		[Fact]
		public async Task GetLatestPosts_WithCountOfZero_ReturnsCollectionOfTenBlogPosts() {
			var logger = Substitute.For<ILogger<BlogController>>();
			var repo = Substitute.For<IBlogRepository>();

			repo.GetLatestPostsAsync(10).Returns(new BlogPost[10]);

			var expectation = new BlogPost[10];

			var controller = new BlogController(logger, repo);
			(await controller.GetLatestPosts()).Value.Should().BeEquivalentTo(expectation);
		}

		[Fact]
		public async Task GetLatestPosts_WithCountOf1_ReturnsCollectionOf1BlogPosts() {
			var logger = Substitute.For<ILogger<BlogController>>();
			var repo = Substitute.For<IBlogRepository>();

			repo.GetLatestPostsAsync(1).Returns(new BlogPost[1]);

			var expectation = new BlogPost[1];

			var controller = new BlogController(logger, repo);
			(await controller.GetLatestPosts(1)).Value.Should().BeEquivalentTo(expectation);
		}

		[Fact]
		public async Task GetLatestPosts_WithCountOf1AndOffsetOf1_ReturnsCollectionOf1BlogPosts() {
			var logger = Substitute.For<ILogger<BlogController>>();
			var repo = Substitute.For<IBlogRepository>();

			repo.GetLatestPostsAsync(1, 1).Returns(new BlogPost[1]);

			var expectation = new BlogPost[1];

			var controller = new BlogController(logger, repo);
			(await controller.GetLatestPosts(1, 1)).Value.Should().BeEquivalentTo(expectation);
		}

		[Fact]
		public async Task GetLatestPosts_WithCountOf1AndOffsetOfOutOfRange_ReturnsCollectionOf1BlogPosts() {
			var logger = Substitute.For<ILogger<BlogController>>();
			var repo = Substitute.For<IBlogRepository>();

			repo.GetLatestPostsAsync(1, 1000).Throws(new ItemNotFoundException("", null));

			var controller = new BlogController(logger, repo);
			(await controller.GetLatestPosts(1, 1000)).Result.Should().BeOfType<NotFoundObjectResult>();
		}
	}
}
